from typing import Any, TypeVar, Union
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field

from ..types import UNSET, Unset

T = TypeVar("T", bound="AccessSubnet")


@_attrs_define
class AccessSubnet:
    """
    Attributes:
        uuid (UUID):
        inet (str):
        customer (str):
        description (Union[Unset, str]):
    """

    uuid: UUID
    inet: str
    customer: str
    description: Union[Unset, str] = UNSET
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        uuid = str(self.uuid)

        inet = self.inet

        customer = self.customer

        description = self.description

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "uuid": uuid,
                "inet": inet,
                "customer": customer,
            }
        )
        if description is not UNSET:
            field_dict["description"] = description

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        uuid = UUID(d.pop("uuid"))

        inet = d.pop("inet")

        customer = d.pop("customer")

        description = d.pop("description", UNSET)

        access_subnet = cls(
            uuid=uuid,
            inet=inet,
            customer=customer,
            description=description,
        )

        access_subnet.additional_properties = d
        return access_subnet

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
