import datetime
from typing import Any, TypeVar
from uuid import UUID

from attrs import define as _attrs_define
from attrs import field as _attrs_field
from dateutil.parser import isoparse

T = TypeVar("T", bound="GroupInvitation")


@_attrs_define
class GroupInvitation:
    """
    Attributes:
        scope_uuid (UUID):
        scope_name (str):
        scope_type (str):
        customer_uuid (UUID):
        customer_name (str):
        role_name (str):
        role_description (str):
        created_by_full_name (str):
        created_by_username (str):
        url (str):
        uuid (UUID):
        role (UUID):
        created (datetime.datetime):
        expires (datetime.datetime):
        is_active (bool):
    """

    scope_uuid: UUID
    scope_name: str
    scope_type: str
    customer_uuid: UUID
    customer_name: str
    role_name: str
    role_description: str
    created_by_full_name: str
    created_by_username: str
    url: str
    uuid: UUID
    role: UUID
    created: datetime.datetime
    expires: datetime.datetime
    is_active: bool
    additional_properties: dict[str, Any] = _attrs_field(init=False, factory=dict)

    def to_dict(self) -> dict[str, Any]:
        scope_uuid = str(self.scope_uuid)

        scope_name = self.scope_name

        scope_type = self.scope_type

        customer_uuid = str(self.customer_uuid)

        customer_name = self.customer_name

        role_name = self.role_name

        role_description = self.role_description

        created_by_full_name = self.created_by_full_name

        created_by_username = self.created_by_username

        url = self.url

        uuid = str(self.uuid)

        role = str(self.role)

        created = self.created.isoformat()

        expires = self.expires.isoformat()

        is_active = self.is_active

        field_dict: dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "scope_uuid": scope_uuid,
                "scope_name": scope_name,
                "scope_type": scope_type,
                "customer_uuid": customer_uuid,
                "customer_name": customer_name,
                "role_name": role_name,
                "role_description": role_description,
                "created_by_full_name": created_by_full_name,
                "created_by_username": created_by_username,
                "url": url,
                "uuid": uuid,
                "role": role,
                "created": created,
                "expires": expires,
                "is_active": is_active,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: type[T], src_dict: dict[str, Any]) -> T:
        d = src_dict.copy()
        scope_uuid = UUID(d.pop("scope_uuid"))

        scope_name = d.pop("scope_name")

        scope_type = d.pop("scope_type")

        customer_uuid = UUID(d.pop("customer_uuid"))

        customer_name = d.pop("customer_name")

        role_name = d.pop("role_name")

        role_description = d.pop("role_description")

        created_by_full_name = d.pop("created_by_full_name")

        created_by_username = d.pop("created_by_username")

        url = d.pop("url")

        uuid = UUID(d.pop("uuid"))

        role = UUID(d.pop("role"))

        created = isoparse(d.pop("created"))

        expires = isoparse(d.pop("expires"))

        is_active = d.pop("is_active")

        group_invitation = cls(
            scope_uuid=scope_uuid,
            scope_name=scope_name,
            scope_type=scope_type,
            customer_uuid=customer_uuid,
            customer_name=customer_name,
            role_name=role_name,
            role_description=role_description,
            created_by_full_name=created_by_full_name,
            created_by_username=created_by_username,
            url=url,
            uuid=uuid,
            role=role,
            created=created,
            expires=expires,
            is_active=is_active,
        )

        group_invitation.additional_properties = d
        return group_invitation

    @property
    def additional_keys(self) -> list[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
